import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class Curtain extends PApplet {// ------------------------------------------------------------------------------
// Curtain - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Curtain.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

// Initiate

int numc = 0;                                                  // Number of string segments we are currently at.
int numObstacles = 1;                                          // Number of obstacles.
int numStrings = 10;                                           // Number of strings.
int numStringSegments = 15;                                    // Number of segments in each string.
int numSegments = numStrings*numStringSegments;                // Number of total segments.

float maxd = 25;                                               // The distance at which the segments feel most comfortable with to each other.
float g = 1.0f;                                                 // Gravity constant.
float k = 0.99f;                                                // Damping constant.
float r = 50;                                                  // Obstacle radius.
float v = 0.9f;                                                 // Obstacle damping.
float m = 30;                                                  // Mouse constant

string[] strings;
stringSegment[] stringSegments;
obstacle[] obstacles;

public void setup()
{
  size(800,420);
  framerate(40);
  smooth();
  reset();
}

public void reset() 
{
  // Create all objects
  
  // Create obstacles
  obstacles = new obstacle[numObstacles];
  for ( int i=0; i<numObstacles; i++ ) { obstacles[i] = new obstacle(); }
  
  // Create strings
  strings = new string[numStrings];
  stringSegments = new stringSegment[numSegments];
  for ( int i=0; i<numStrings; i++ ) { strings[i] = new string(); }
}

public void draw()
{
  background(255,255,255);
  
  // Redraw/Recalculate all objects where necessary
 
  for ( int i=0; i<numStrings; i++ )
  {
    strings[i].drawString(width/2+15*(i-numStrings/2), 0,-1,-1);
  }

  if ( mouseX == 0 && mouseX == 0 )
  {
    obstacles[0].obstaclate(-r,-r);
  }
  else
  {
    obstacles[0].obstaclate(mouseX,mouseY);
  }
  
  stroke(0,0,0);
  strokeWeight(3);
  line(300,0,500,0);
}

// ------------------------------------------------------------------------------
// Curtain - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Obstacle.java
// These objects are are obstacles for the
// string. They block their movement.
// ------------------------------------------------------------------------------

class obstacle
{
  float fx;
  float fy;
  float vx;
  float vy;
  float x;
  float y;

  obstacle()
  {
    x = 400;
    y = 210;
  }
  
  public void obstaclate(float X, float Y)
  {
    move(X,Y);
    block();
    display();
  }
  
  public void block()
  {
    fx = 0;
    fy = 0;
    
    for ( int i=0; i<numSegments; i++ )
    {
      float deltax = x-stringSegments[i].x;
      float deltay = y-stringSegments[i].y;
      float theta = atan2(deltay,deltax);
      float d = sqrt(pow(deltax,2)+pow(deltay,2));
      
      if ( d < r )
      {
        stringSegments[i].assignPosition(x-r*cos(theta),y-r*sin(theta));
        stringSegments[i].bx = -(r-d)*cos(theta);
        stringSegments[i].by = -(r-d)*sin(theta);
        
        fx = (r-d)*cos(theta);
        fy = (r-d)*sin(theta);
      }
    }
  }
  
  public void move(float X, float Y)
  {
    float deltax;
    float deltay;
    
    if (mouseY != 0 || mouseY != 0)
    {
      deltax = X-x;
      deltay = Y-y;
    }
    else
    {
      deltax = 400-x;
      deltay = 210-y;
    }
    
    fx += deltax/m;
    fy += deltay/m;
    
    vx += fx/10;
    vy += fy/10;
    
    x += vx;
    y += vy;
    
    // damping
    vx *= v;
    vy *= v;
  }
  
  public void display()
  {
    stroke(0,0,0);
    fill(240,240,240);
    ellipse(x,y,2*r,2*r);
    
    if (mouseX != 0 || mouseY != 0)
    {
      stroke(200,200,200);
      line(x,y,mouseX,mouseY);
      strokeWeight(10);
      stroke(50,50,50);
      point(x,y);
      point(mouseX, mouseY);
    }
  }
}

// ------------------------------------------------------------------------------
// Curtain - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// String.java
// These objects are the controllers for each string
// segment. They create them, tell them where to start off from
// and where to end.
// ------------------------------------------------------------------------------

class string
{
  int n0;
  
  string()
  {
    n0 = numc;

    for ( int i=0; i<numStringSegments; i++ )
    {
      stringSegments[numc] = new stringSegment(numc);
      numc++;
    }
  }
  
  public void drawString(float X1, float Y1, float X2, float Y2)
  {
    for ( int i=0; i<numStringSegments; i++ )
    {
      if ( i > 0 && i < numStringSegments - 1 )
      {
        stringSegments[n0+i].fall();
        stringSegments[n0+i].pull();
        stringSegments[n0+i].move();
        stringSegments[n0+i].drawSelf();
        stringSegments[n0+i].drawLine();
      }
      else
      {
        if ( i == 0 )
        {
            stringSegments[n0+i].assignPosition(X1,Y1);
            stringSegments[n0+i].pull();
            stringSegments[n0+i].drawSelf();
        }
        else
        {
          if ( X2 == -1 && Y2 == -1 )
          {
            stringSegments[n0+i].fall();
            stringSegments[n0+i].move();
            stringSegments[n0+i].drawSelf();
            stringSegments[n0+i].drawLine();
            stringSegments[n0+i].drawBob();
          }
          else
          {
            stringSegments[n0+i].assignPosition(X2,Y2);
            stringSegments[n0+i].drawSelf();
            stringSegments[n0+i].drawLine();
          }
        }
      }
    }
  }
}

// ------------------------------------------------------------------------------
// Curtain - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// String.java
// These objects are the actual small straight
// lines that make up the whole string. All the physics
// is applied to each and everyone of them.
// ------------------------------------------------------------------------------

class stringSegment
{
  float x;
  float y;
  
  float tx;        // Tension in string
  float ty;
  float wx;        // Weight of gravity
  float wy;
  float fx;        // Total force
  float fy;
  float bx;        // Bounce force - Collisions
  float by;
  
  float vx;
  float vy;
  float m;
  int n;
  
  stringSegment(int N)
  {
    x = width/2 + random(-700,700);
    y = 0;
    vx = random(-40,40);
    vy = 0;
    n = N;
    m = 1;
  }

  public void drawSelf()
  {
    //fill(255,255,255);
    //stroke(200,200,200);
    //ellipse(x,y,6,6);
  }
  
  public void drawLine()
  {
    stroke(50,50,50);
    strokeWeight(2);
    line(stringSegments[n-1].x,stringSegments[n-1].y,x,y);
  }
  
  public void drawBob()
  {
    fill(230,230,230);
    stroke(0,0,0);
    ellipse(x,y,10,10);
  }
  
  public void pull()
  {
    float deltax = x-stringSegments[n+1].x;
    float deltay = y-stringSegments[n+1].y;
    float d = sqrt(pow(deltax,2)+pow(deltay,2));
    float theta = atan2(deltay,deltax);
    float ft = (maxd-d)/2;

    tx += ft*cos(theta);
    ty += ft*sin(theta);
    
    stringSegments[n+1].assignPosition(x-maxd*cos(theta),y-maxd*sin(theta)); // Makes strings rigid - remove for elasticity
    stringSegments[n+1].tx = -ft*cos(theta);
    stringSegments[n+1].ty = -ft*sin(theta);
    stringSegments[n+1].vx *= k;
    stringSegments[n+1].vy *= k;
  }
  
  public void fall()
  {
    // Vertical force of gravity
    wx = 0;
    wy = m*g;
  }
 
  public void move()
  {
    // Calculate the total force
    fx = tx + wx + bx;
    fy = ty + wy + by;
      
    // Update velocity with acceleration
    vx += fx/m;
    vy += fy/m;
      
    // Update positions
    x += vx;
    y += vy;
    
    // Reset b force
    bx = 0;
    by = 0;
  }
  
  public void assignPosition(float X, float Y)
  {
    x = X;
    y = Y;
  }
}
static public void main(String args[]) {   PApplet.main(new String[] { "Curtain" });}}