// ------------------------------------------------------------------------------
// Curtain - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// String.java
// These objects are the actual small straight
// lines that make up the whole string. All the physics
// is applied to each and everyone of them.
// ------------------------------------------------------------------------------

class stringSegment
{
  float x;
  float y;
  
  float tx;        // Tension in string
  float ty;
  float wx;        // Weight of gravity
  float wy;
  float fx;        // Total force
  float fy;
  float bx;        // Bounce force - Collisions
  float by;
  
  float vx;
  float vy;
  float m;
  int n;
  
  stringSegment(int N)
  {
    x = width/2 + random(-700,700);
    y = 0;
    vx = random(-40,40);
    vy = 0;
    n = N;
    m = 1;
  }

  void drawSelf()
  {
    //fill(255,255,255);
    //stroke(200,200,200);
    //ellipse(x,y,6,6);
  }
  
  void drawLine()
  {
    stroke(50,50,50);
    strokeWeight(2);
    line(stringSegments[n-1].x,stringSegments[n-1].y,x,y);
  }
  
  void drawBob()
  {
    fill(230,230,230);
    stroke(0,0,0);
    ellipse(x,y,10,10);
  }
  
  void pull()
  {
    float deltax = x-stringSegments[n+1].x;
    float deltay = y-stringSegments[n+1].y;
    float d = sqrt(pow(deltax,2)+pow(deltay,2));
    float theta = atan2(deltay,deltax);
    float ft = (maxd-d)/2;

    tx += ft*cos(theta);
    ty += ft*sin(theta);
    
    stringSegments[n+1].assignPosition(x-maxd*cos(theta),y-maxd*sin(theta)); // Makes strings rigid - remove for elasticity
    stringSegments[n+1].tx = -ft*cos(theta);
    stringSegments[n+1].ty = -ft*sin(theta);
    stringSegments[n+1].vx *= k;
    stringSegments[n+1].vy *= k;
  }
  
  void fall()
  {
    // Vertical force of gravity
    wx = 0;
    wy = m*g;
  }
 
  void move()
  {
    // Calculate the total force
    fx = tx + wx + bx;
    fy = ty + wy + by;
      
    // Update velocity with acceleration
    vx += fx/m;
    vy += fy/m;
      
    // Update positions
    x += vx;
    y += vy;
    
    // Reset b force
    bx = 0;
    by = 0;
  }
  
  void assignPosition(float X, float Y)
  {
    x = X;
    y = Y;
  }
}
